@@ -2,6 +2,7 @@ from __future__ import absolute_import
 
 import time
 import string
+import logging
 from contextlib import contextmanager
 
 import xapian
@@ -18,6 +19,8 @@ from .exc import ValidationError
 
 RETRY_LIMIT = 3
 
+logger = logging.getLogger(__name__)
+
 
 default_parser_flags = (QueryParser.FLAG_PHRASE |
                         QueryParser.FLAG_BOOLEAN |
@@ -182,6 +185,21 @@ class Database(object):
             self.cancel()
             raise
 
+    def retry_if_modified(self, operation, limit=RETRY_LIMIT):
+        tries = 0
+        while True:
+            try:
+                return operation()
+            except xapian.DatabaseModifiedError:
+                if tries > limit:
+                    logger.warning(
+                        'Database operation failed after %s retries.',
+                        tries)
+                    raise
+                logger.info('Retrying database operation.')
+                self.reopen()
+                tries += 1
+
     def __init__(self, path,
                  writable=True,
                  overwrite=False,
@@ -579,7 +597,9 @@ class Database(object):
                 if translit:
                     query = query.encode(translit)
                 qp = self.prepare_query_parser(language, default_op)
-                return qp.parse_query(query, parser_flags)
+                op = lambda: qp.parse_query(query, parser_flags)
+                return self.retry_if_modified(op)
+
         else:
             return reduce(partial(Query, default_op),
                           (self.querify(q,
@@ -771,8 +791,8 @@ class Database(object):
         Describe the parsed query.
         """
         qp = self.prepare_query_parser(language, default_op)
-        q = qp.parse_query(query, default_parser_flags)
-        return str(q)
+        op = lambda: qp.parse_query(query, default_parser_flags) 
+        return str(self.retry_if_modified(op))
 
     def spell(self, query,
               language=None):
@@ -781,7 +801,8 @@ class Database(object):
         """
         self.backend.reopen()
         qp = self.prepare_query_parser(language)
-        qp.parse_query(query, QueryParser.FLAG_SPELLING_CORRECTION)
+        op = lambda: qp.parse_query(query, QueryParser.FLAG_SPELLING_CORRECTION)
+        self.retry_if_modified(op)
         return qp.get_corrected_query_string().decode('utf8')
 
     def suggest(self, query,
@@ -872,18 +893,9 @@ class Database(object):
         if check is None:
             check = limit + 1
 
-        tries = 0
-        while True:
-            try:
-                mset = enq.get_mset(
-                    offset, limit, check, None, match_decider, match_spy)
-                break
-            except xapian.DatabaseModifiedError:
-                if tries > RETRY_LIMIT:
-                    raise
-                self.reopen()
-                tries += 1
-        return mset
+        op = lambda: enq.get_mset(
+                offset, limit, check, None, match_decider, match_spy)
+        return self.retry_if_modified(op)
 
     def _return_objects(self, mset, document=False):
         for record in mset:
@@ -894,16 +906,8 @@ class Database(object):
             if document:
                 yield doc
             else:
-                tries = 0
-                while True:
-                    try:
-                        data = doc.get_data()
-                        break
-                    except xapian.DatabaseModifiedError:
-                        if tries > RETRY_LIMIT:
-                            raise
-                        self.reopen()
-                        tries += 1
+                op = lambda: doc.get_data()
+                data = self.retry_if_modified(op)
                 typ, data = loads(data)
                 yield self.record_factory(_lookup_schema(typ).from_flat(data))
 
